"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = viteMetaEnvBabelPlugin;
const REPLACE_VARS = [{
  regex: /^VITE_/,
  replacement: (template, variableName) => template.expression('process.env.%%variableName%%')({
    variableName
  })
}, {
  regex: /^(NODE_ENV|MODE)$/,
  replacement: template => template.expression.ast("process.env.NODE_ENV || 'test'")
}, {
  regex: /^BASE_URL$/,
  replacement: template => template.expression.ast("'/'")
}, {
  regex: /^DEV$/,
  replacement: template => template.expression.ast("process.env.NODE_ENV !== 'production'")
}, {
  regex: /^PROD$/,
  replacement: template => template.expression.ast("process.env.NODE_ENV === 'production'")
}, {
  regex: /.*/,
  replacement: template => template.expression.ast('undefined')
}];

function getReplacement(variableName, template) {
  return REPLACE_VARS.filter(({
    regex
  }) => regex.test(variableName)).map(({
    replacement
  }) => replacement(template, variableName))[0];
}

function viteMetaEnvBabelPlugin({
  template,
  types: t
}) {
  return {
    name: 'vite-meta-env',
    visitor: {
      MemberExpression(path) {
        const envNode = t.isMemberExpression(path.node.object) && path.node.object;
        const variableName = t.isIdentifier(path.node.property) && path.node.property.name;

        if (!envNode || !variableName) {
          return;
        }

        const isMetaProperty = t.isMetaProperty(envNode.object);
        const isEnvVar = t.isIdentifier(envNode.property) && envNode.property.name === 'env';

        if (!isMetaProperty || !isEnvVar) {
          return;
        }

        const replacement = getReplacement(variableName, template);
        path.replaceWith(replacement);
      }

    }
  };
}